#include <iostream>
#include "String.h"

using namespace std;

template <class T>
class Element {
public:
	void show();

	T *getData();
	void setData(T *input_data);

	Element *getNext();
	void setNext(Element *input_next);

private:
	T *data;
	Element *next;
};

template <class T>
class Dynamo {
public:
	Dynamo();
	~Dynamo();

	void add(T data);
	bool remove(Element<T> *elem);
	void edit(Element<T> *elem, T data);
	void show();

	Element<T> *get(const char *str);
	Element<T> *get(const int value);

	void sort();
	void sortByName();

private:
	Element<T> *head;
};

template <class T>
Dynamo<T>::Dynamo()
{
	head = NULL;
}

template <class T>
Dynamo<T>::~Dynamo()
{
	while (head != NULL) {
		Element<T> *temp = head->getNext();
		delete head;
		head = temp;
	}
}

template <class T>
void Dynamo<T>::add(T data)
{
	Element<T> *temp = new Element<T>;
	temp->setData(new T(data));
	temp->setNext(head);
	head = temp;
}

template <class T>
bool Dynamo<T>::remove(Element<T> *elem)
{
	Element<T> *temp = head;

	if (temp == elem) {
		head = elem->getNext();
		return true;
	}

	while (temp != NULL) {
		if (temp->getNext() == elem) {
			temp->setNext(elem->getNext());
			delete elem;
			return true;
		}
		temp = temp->getNext();
	}
	return false;
}

template <class T>
void Dynamo<T>::edit(Element<T> *elem, T data)
{
	Element<T> *temp = new Element<T>;
	temp->setData(new T(data));
	temp->setNext(elem->getNext());
	*elem = *temp;
}

template <class T>
void Dynamo<T>::show()
{
	Element<T> *temp = head;

	while (temp != NULL) {
		cout << *temp->getData() << endl;
		temp = temp->getNext();
	}
}

template <class T>
Element<T> *Dynamo<T>::get(const int value)
{
	Element<T> *temp = head;

	while (temp != NULL) {
		if (temp->data == value) {
			return temp;
		}
		temp = temp->next;
	}
	return NULL;
}

template <class T>
Element<T> *Dynamo<T>::get(const char *str)
{
	Element<T> *temp = head;

	while (temp != NULL) {
		if (temp->data == str) {
			return temp;
		}
		temp = temp->next;
	}
	return NULL;
}

template <class T>
void Dynamo<T>::sort()
{
	if (head) {
		Element<T> *res = head;
		head = head->next;
		res->next = NULL;

		while (head) {
			Element<T> *elem = head;
			head = head->next;

			if (*elem->data < *res->data) {
				elem->next = res;
				res = elem;
			} else {
				Element<T> *p = res;

				while (p->next) {
					if (*p->next->data > *elem->data) {
						break;
					}

					p = p->next;
				}

				elem->next = p->next;
				p->next = elem;
			}
		}
		head = res;
	}
}

template <class T>
T *Element<T>::getData()
{
    return data;
}

template <class T>
void Element<T>::setData(T *input_data)
{
    data = input_data;
}

template <class T>
Element<T> *Element<T>::getNext()
{
    return next;
}

template <class T>
void Element<T>::setNext(Element *input_next)
{
    next = input_next;
}
